import os
import sys


sys.path.insert(1, os.path.join(sys.path[0], ".."))

from orso import DataFrame
from tests.cities import values, schema


def test_group_by_sum_population():
    df = DataFrame(values)
    result = df.group_by("language").sum("population")

    expected_result = {
        "Japanese": 13929286,
        "English": 37606863,
        "Hindi, English": 18500000,
        "Afrikaans, English": 433688,
        "French": 2148000,
    }

    for row in result:
        row = row.as_dict
        if row["language"] in expected_result:
            assert row["SUM(population)"] == expected_result[row["language"]], row


def test_group_by_min_population():
    df = DataFrame(values)
    result = df.group_by("country").min("population")

    assert len(result) == 20

    for row in result:
        row_dict = row.as_dict
        if row_dict["country"] == "United States":
            assert row_dict["MIN(population)"] == 8399000


def test_group_by_max_population():
    df = DataFrame(values)
    result = df.group_by("country").max("population")

    assert len(result) == 20

    for row in result:
        row_dict = row.as_dict
        if row_dict["country"] == "United Kingdom":
            assert row_dict["MAX(population)"] == 8982000


def test_group_by_avg_population():
    df = DataFrame(values)
    result = df.group_by("language").avg("population")

    assert len(result) == 17

    for row in result:
        row_dict = row.as_dict
        if row_dict["language"] == "English":
            assert (
                row_dict["AVG(population)"] == 9401715.75
            )  # (8399000 + 5312163 + 14913700, 8982000) / 4


def test_group_by_count_population():
    df = DataFrame(values)
    result = df.group_by("language").count()

    assert len(result) == 17

    for row in result:
        row_dict = row.as_dict
        if row_dict["language"] == "English":
            assert row_dict["COUNT(*)"] == 4


def test_group_by_groups_language():
    df = DataFrame(values)
    result = df.group_by("language").groups()

    assert len(result) == 17


if __name__ == "__main__":  # prgama: nocover
    from tests import run_tests

    test_group_by_count_population()
    run_tests()



import random

first_names_male = (
    "David",
    "Paul",
    "Christopher",
    "Thomas",
    "John",
    "Mark",
    "James",
    "Stephen",
    "Andrew",
    "Jack",
    "Michael",
    "Daniel",
    "Peter",
    "Richard",
    "Matthew",
    "Robert",
    "Ryan",
    "Joshua",
    "Alan",
    "Ian",
    "Simon",
    "Luke",
    "Samuel",
    "Jordan",
    "Anthony",
    "Adam",
    "Lee",
    "Alexander",
    "William",
    "Kevin",
    "Darren",
    "Benjamin",
    "Philip",
    "Gary",
    "Joseph",
    "Brian",
    "Steven",
    "Liam",
    "Keith",
    "Martin",
    "Jason",
    "Jonathan",
    "Jake",
    "Graham",
    "Nicholas",
    "Craig",
    "George",
    "Colin",
    "Neil",
    "Lewis",
    "Nigel",
    "Oliver",
    "Timothy",
    "Stuart",
    "Kenneth",
    "Raymond",
    "Jamie",
    "Nathan",
    "Geoffrey",
    "Connor",
    "Terence",
    "Trevor",
    "Adrian",
    "Harry",
    "Malcolm",
    "Scott",
    "Callum",
    "Wayne",
    "Aaron",
    "Barry",
    "Ashley",
    "Bradley",
    "Patrick",
    "Gareth",
    "Jacob",
    "Sean",
    "Kieran",
    "Derek",
    "Carl",
    "Dean",
    "Charles",
    "Sam",
    "Shaun",
    "Ben",
    "Roger",
    "Mohammed",
    "Leslie",
    "Ronald",
    "Kyle",
    "Clive",
    "Edward",
    "Antony",
    "Jeremy",
    "Justin",
    "Jeffrey",
    "Christian",
    "Roy",
    "Karl",
    "Alex",
    "Gordon",
    "Dominic",
    "Joe",
    "Marc",
    "Reece",
    "Dennis",
    "Russell",
    "Gavin",
    "Rhys",
    "Phillip",
    "Allan",
    "Robin",
    "Charlie",
    "Gerald",
    "Ross",
    "Francis",
    "Eric",
    "Julian",
    "Bernard",
    "Dale",
    "Donald",
    "Damian",
    "Frank",
    "Shane",
    "Cameron",
    "Norman",
    "Duncan",
    "Louis",
    "Frederick",
    "Tony",
    "Howard",
    "Conor",
    "Douglas",
    "Garry",
    "Elliot",
    "Marcus",
    "Arthur",
    "Vincent",
    "Max",
    "Mathew",
    "Abdul",
    "Henry",
    "Martyn",
    "Ricky",
    "Leonard",
    "Lawrence",
    "Glen",
    "Mitchell",
    "Gerard",
    "Gregory",
    "Iain",
    "Billy",
    "Bryan",
    "Joel",
    "Clifford",
    "Josh",
    "Leon",
    "Stewart",
    "Mohammad",
    "Dylan",
    "Graeme",
    "Terry",
    "Guy",
    "Elliott",
    "Stanley",
    "Danny",
    "Brandon",
    "Victor",
    "Toby",
    "Hugh",
    "Mohamed",
    "Brett",
    "Albert",
    "Tom",
    "Declan",
    "Maurice",
    "Glenn",
    "Leigh",
    "Denis",
    "Damien",
    "Bruce",
    "Jay",
    "Owen",
)

first_names_female = (
    "Susan",
    "Sarah",
    "Rebecca",
    "Linda",
    "Julie",
    "Claire",
    "Laura",
    "Lauren",
    "Christine",
    "Karen",
    "Nicola",
    "Gemma",
    "Jessica",
    "Margaret",
    "Jacqueline",
    "Emma",
    "Charlotte",
    "Janet",
    "Deborah",
    "Lisa",
    "Hannah",
    "Patricia",
    "Tracey",
    "Joanne",
    "Sophie",
    "Carol",
    "Jane",
    "Michelle",
    "Victoria",
    "Amy",
    "Elizabeth",
    "Helen",
    "Samantha",
    "Emily",
    "Mary",
    "Diane",
    "Rachel",
    "Anne",
    "Sharon",
    "Ann",
    "Tracy",
    "Amanda",
    "Jennifer",
    "Chloe",
    "Angela",
    "Louise",
    "Katie",
    "Lucy",
    "Barbara",
    "Alison",
    "Sandra",
    "Caroline",
    "Clare",
    "Kelly",
    "Bethany",
    "Gillian",
    "Natalie",
    "Jade",
    "Pauline",
    "Megan",
    "Elaine",
    "Alice",
    "Lesley",
    "Catherine",
    "Hayley",
    "Pamela",
    "Danielle",
    "Holly",
    "Wendy",
    "Abigail",
    "Valerie",
    "Olivia",
    "Jean",
    "Dawn",
    "Donna",
    "Stephanie",
    "Leanne",
    "Kathleen",
    "Natasha",
    "Denise",
    "Sally",
    "Katherine",
    "Georgia",
    "Maureen",
    "Maria",
    "Zoe",
    "Judith",
    "Kerry",
    "Debra",
    "Melanie",
    "Stacey",
    "Eleanor",
    "Paula",
    "Shannon",
    "Sheila",
    "Joanna",
    "Paige",
    "Janice",
    "Lorraine",
    "Georgina",
    "Lynn",
    "Andrea",
    "Suzanne",
    "Nicole",
    "Yvonne",
    "Chelsea",
    "Lynne",
    "Anna",
    "Kirsty",
    "Shirley",
    "Alexandra",
    "Marion",
    "Beverley",
    "Melissa",
    "Rosemary",
    "Kimberley",
    "Carole",
    "Fiona",
    "Kate",
    "Joan",
    "Marie",
    "Jenna",
    "Marilyn",
    "Jodie",
    "June",
    "Grace",
    "Mandy",
    "Rachael",
    "Lynda",
    "Tina",
    "Kathryn",
    "Molly",
    "Jayne",
    "Amber",
    "Marian",
    "Jasmine",
    "Brenda",
    "Sara",
    "Kayleigh",
    "Teresa",
    "Harriet",
    "Julia",
    "Ashleigh",
    "Heather",
    "Kim",
    "Ruth",
    "Jemma",
    "Carly",
    "Leah",
    "Eileen",
    "Francesca",
    "Naomi",
    "Hilary",
    "Abbie",
    "Sylvia",
    "Katy",
    "Irene",
    "Cheryl",
    "Rosie",
    "Dorothy",
    "Aimee",
    "Vanessa",
    "Ellie",
    "Frances",
    "Sian",
    "Josephine",
    "Gail",
    "Jill",
    "Lydia",
    "Joyce",
    "Charlene",
    "Hollie",
    "Hazel",
    "Annette",
    "Bethan",
    "Amelia",
    "Beth",
    "Rita",
    "Geraldine",
    "Diana",
    "Lindsey",
    "Carolyn",
)

# fmt: off
surname = (
    "Smith", "Johnson", "Williams", "Brown", "Jones", "Garcia", "Miller",
    "Davis", "Rodriguez", "Martinez", "Hernandez", "Lopez", "Gonzalez",
    "Wilson", "Anderson", "Thomas", "Taylor", "Moore", "Jackson", "Martin",
    "Lee", "Perez", "Thompson", "White", "Harris", "Sanchez", "Clark",
    "Ramirez", "Lewis", "Robinson", "Walker", "Young", "Allen", "King",
    "Wright", "Scott", "Torres", "Nguyen", "Hill", "Flores", "Green",
    "Adams", "Nelson", "Baker", "Hall", "Rivera", "Campbell", "Mitchell",
    "Carter", "Roberts", "Gomez", "Phillips", "Evans", "Turner", "Diaz",
    "Parker", "Cruz", "Edwards", "Collins", "Valentine", "Rich", "Boyd"
    "Sullivan", "Gonzales", "Peters", "Fox", "Ray", "Bradley", "Jensen",
    "Burke", "Black", "Dunn", "Hunter", "Fields", "Owen", "Riley",
    "Goodwin", "Banks", "Gilbert", "Cain", "Nash", "Stevens", "Ford",
    "Cross", "Moss", "Hopkins", "Curry", "McLaughlin", "Harmon",
    "Monroe", "Vincent", "Perry", "Shields", "Kerr", "Townsend",
    "Blake", "Glover", "Pope", "Schwartz", "Lamb", "Manning", "Walter",
    "Leonard", "Dean", "Conway", "Boone", "Church", "Morse", "Dodson",
    "Flynn", "Shea", "Sheppard", "Osborne", "Cochran", "Prince", "Case",
    "Barr", "Rosales", "York", "Winters", "McLean", "Giles", "McKee",
    "Glass", "Dickson", "Hancock", "Gaines", "Fry", "Skinner", "Wiggins",
    "Cameron", "Clements", "Christensen", "Klein", "Pratt", "Briggs",
    "Osborn", "Bond", "Rush", "Churchill", "Corbin", "Leblanc", "Daley",
    "Mullen", "Pham", "Charles", "Graves", "Justice", "Mayer", "Donald",
    "Lyons", "Hines", "Gallagher", "Dorsey", "Shepherd", "Kaiser",
    "Cline", "Joyce", "Hawkins", "Montgomery", "Daugherty", "Hoover",
    "Baird", "Wiley", "Cooke", "Holt", "Duffy", "Buckley", "Haley",
)
# fmt: on

formats = (
    "{first_names_male} {surname}",
    "{first_names_male} {surname}",
    "{first_names_male} {surname}",
    "{first_names_male} {surname}",
    "{first_names_female} {surname}",
    "{first_names_female} {surname}",
    "{first_names_female} {surname}",
    "{first_names_female} {surname}",
    "{first_names_male} {surname}-{second_surname}",
    "{first_names_female} {surname}-{second_surname}",
)


def generate_random_name() -> str:
    """
    Generates a random name based on the given name tuples and formats.

    Returns:
        str: A randomly generated name.
    """
    selected_format = random.choice(formats)
    placeholders = {
        "first_names_male": (
            random.choice(first_names_male) if "{first_names_male}" in selected_format else ""
        ),
        "first_names_female": (
            random.choice(first_names_female) if "{first_names_female}" in selected_format else ""
        ),
        "surname": random.choice(surname) if "{surname}" in selected_format else "",
        "second_surname": random.choice(surname) if "{second_surname}" in selected_format else "",
    }

    return selected_format.format(**placeholders)



"""
https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry
"""

import atexit
import datetime
import logging
import os
from typing import Any
from typing import Dict
from typing import Optional
from typing import Union

import orjson as json

from orso.logging.levels import LEVELS
from orso.logging.levels import LEVELS_TO_STRING
from orso.logging.log_formatter import LogFormatter

logging_seen_warnings: Dict[int, int] = {}


def fix_fields(obj: Any) -> Any:
    if isinstance(obj, (datetime.date, datetime.datetime)):
        return obj.isoformat()
    if isinstance(obj, bytes):
        return obj.decode("UTF8")
    if isinstance(obj, dict):
        return {k: fix_fields(v) for k, v in obj.items()}
    return str(obj)


def fix_dict(obj: dict) -> dict:
    return {k: fix_fields(v) for k, v in obj.items()}


def report_suppressions(message):
    from .. import logging as ml

    record = logging_seen_warnings.get(hash(message))
    if record:
        ml.get_logger().warning(
            f'The following message was suppressed {record} time(s) - "{message}"'
        )


def log_it(payload):
    if isinstance(payload, dict):
        payload = json.dumps(fix_dict(payload))
    if isinstance(payload, bytes):
        payload = payload.decode()
    print(payload, flush=True)
    return payload


def extract_caller():
    import os.path
    import traceback

    frames = traceback.extract_stack()
    if len(frames) < 4:
        return "<unknown>", "<unknown>", -1
    frame = frames[len(frames) - 4]
    head, tail = os.path.split(frame.filename)
    return frame.name, tail, frame.lineno


class GoogleLogger(object):
    @staticmethod
    def supported():
        if os.environ.get("K_SERVICE", "") or "" != "":  # nosemgrep
            return True
        return False

    @staticmethod
    def write_event(
        message: Union[str, dict],
        system: Optional[str] = None,
        severity: Optional[int] = logging.DEBUG,
        spanId: Optional[str] = None,
    ):
        # supress duplicate warnings
        if severity == LEVELS.WARNING:  # warnings
            hashed = hash(str(message))
            if hashed in logging_seen_warnings:
                logging_seen_warnings[hashed] += 1
                return "suppressed"
            logging_seen_warnings[hashed] = 0
            atexit.register(report_suppressions, str(message))

        structured_log = {
            "severity": str(severity).split(".")[-1],
            "logging.googleapis.com/labels": {
                "system": system,
                "platform": os.environ.get("LOG_SINK"),
                "formatter": "MABEL",
            },
        }

        method, module, line = extract_caller()
        structured_log["logging.googleapis.com/sourceLocation"] = {
            "function": method,
            "file": module,
            "line": line,
        }  # type:ignore

        if spanId:
            structured_log["logging.googleapis.com/spanId"] = spanId

        if isinstance(message, dict):
            formatter = LogFormatter(None)
            message = formatter.clean_record(message, False)
            structured_log["message"] = str(message) + " *"  # We don't want valid JSON here
            structured_log.update(message)  # type:ignore
            return log_it(structured_log)
        else:
            structured_log["message"] = message
            return log_it(structured_log)

    def __init__(self):
        # rewrite one of the levels
        LEVELS_TO_STRING[LEVELS.AUDIT] = "NOTICE"

        self.level = int(os.environ.get("LOGGING_LEVEL", 25))
        self.debug = self.create_logger(LEVELS.DEBUG)
        self.info = self.create_logger(LEVELS.INFO)
        self.warning = self.create_logger(LEVELS.WARNING)
        self.error = self.create_logger(LEVELS.ERROR)
        self.audit = self.create_logger(LEVELS.AUDIT)
        self.alert = self.create_logger(LEVELS.ALERT)

    def create_logger(self, level):
        from .create_logger import LOG_NAME

        def base_logger(message):
            return GoogleLogger.write_event(message=message, system=LOG_NAME, severity=level)

        def do_nothing(message):
            pass

        if level > self.level:
            return base_logger
        else:
            return do_nothing

    def setLevel(self, level):
        self.level = level
        self.debug = self.create_logger(LEVELS.DEBUG)
        self.info = self.create_logger(LEVELS.INFO)
        self.warning = self.create_logger(LEVELS.WARNING)
        self.error = self.create_logger(LEVELS.ERROR)
        self.audit = self.create_logger(LEVELS.AUDIT)
        self.alert = self.create_logger(LEVELS.ALERT)

    def __call__(self, message):
        self.debug(message)



import platform

from Cython.Build import cythonize
from setuptools import Extension
from setuptools import find_packages
from setuptools import setup

LIBRARY = "orso"


def is_mac():  # pragma: no cover
    return platform.system().lower() == "darwin"


if is_mac():
    COMPILE_FLAGS = ["-O2"]
else:
    COMPILE_FLAGS = ["-O2", "-march=native"]

__version__ = "notset"
with open(f"{LIBRARY}/version.py", mode="r") as v:
    vers = v.read()
exec(vers)  # nosec

with open("README.md", mode="r", encoding="UTF8") as rm:
    long_description = rm.read()

try:
    with open("requirements.txt", "r") as f:
        required = f.read().splitlines()
except:
    with open(f"{LIBRARY}.egg-info/requires.txt", "r") as f:
        required = f.read().splitlines()

extensions = [
    # Compile C++ code first
    Extension(
        name="orso.cityhash.cityhash",
        sources=[
            "orso/cityhash/city.cc",
            "orso/cityhash/cityhash.cpp",
        ],
        language="c++",
    ),
    # Cython code
    Extension(
        name="orso.bitarray.cbitarray",
        sources=["orso/bitarray/cbitarray.pyx"],
        extra_compile_args=COMPILE_FLAGS,
        extra_link_args=COMPILE_FLAGS,
    ),
    Extension(
        name="orso.compiled",
        sources=["orso/compiled.pyx"],
        extra_compile_args=COMPILE_FLAGS,
        extra_link_args=COMPILE_FLAGS,
    ),
]

setup_config = {
    "name": LIBRARY,
    "version": __version__,
    "description": "🐻 DataFrame Library",
    "long_description": long_description,
    "long_description_content_type": "text/markdown",
    "author_email": "justin.joyce@joocer.com",
    "packages": find_packages(include=[LIBRARY, f"{LIBRARY}.*"]),
    "url": "https://github.com/mabel-dev/orso/",
    "ext_modules": cythonize(extensions),
    "install_requires": required,
}

setup(**setup_config)



"""
Bloom Filter

This is a variation of the Bloom Filter; this provides a fast and memory efficient way
to tell if an item is in a list.

This is a probabilistic algorithm, it saves memory and/or time to give you an
approximation of the correct answer. It doesn't claim to be 100% correct 100% of the
time.

https://en.wikipedia.org/wiki/Bloom_filter

(C) 2021-2023 Justin Joyce.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from orso.bitarray import BitArray
from orso.cityhash import CityHash32

HASH_SEEDS = (
    b"ANTHROPOMORPHISM",
    b"BLOODYMINDEDNESS",
    b"CHARACTERIZATION",
    b"CONTEMPTUOUSNESS",
    b"DISFRANCHISEMENT",
    b"DISINGENUOUSNESS",
    b"ELECTROTECHNICAL",
    b"HYPERVENTILATION",
    b"INCOMPREHENSIBLE",
    b"NONRECIPROCATING",
    b"ONEQUINTILLIONTH",
    b"PRESUMPTUOUSNESS",
    b"QUINTESSENTIALLY",
    b"SENSATIONALISTIC",
    b"THREEDIMENSIONAL",
    b"UNCOMPREHENSIBLE",
    b"UNDIPLOMATICALLY",
    b"UNUNDERSTANDABLY",
)


def _log(x):
    return 99999999 * (x ** (1 / 99999999) - 1)


def _get_size(number_of_elements: int, fp_rate: float) -> int:
    """
    Calculate the size of the bitarray

    Parameters:
        number_of_elements: integer
            The number of items expected to be stored in filter
        fp_rate: float (optional)
            False Positive rate (0 to 1), default 0.05

    Returns:
        integer
    """
    size = -(number_of_elements * _log(fp_rate)) / (_log(2) ** 2) + 1
    return int(size)


def _get_hash_count(filter_size, number_of_elements):
    """
    Calculate the number of hashes to use to identify elements

    Parameters:
        filter_size: integer
            The size of the filter bit array
        number_of_elements: integer
            The number of items expected to be stored in filter

    Returns:
        integer
    """
    k = (filter_size / number_of_elements) * _log(2)
    return max(round(k), 2)


class BloomFilter:
    __slots__ = ("filter_size", "bits", "hash_seeds", "hash_count")

    def __init__(self, number_of_elements: int = 50000, fp_rate: float = 0.01):
        """
        Bloom Filters are a probabilistic approach to tracking items in a list.
        They use an array of booleans which are set according to hashes of the
        data items. Items are considered to be in the list if the booleans at
        the hashes are set, this results in a degree of false positives. This
        is factored into the calculation of the size of the boolean array and
        the number of hashes.

        This is used in the profiler to track unique string values without
        having to store the values or hashes of the values (minor errors
        with this count is not expected to be a problem)
        """
        self.filter_size: int = _get_size(number_of_elements, fp_rate)
        self.hash_count: int = _get_hash_count(self.filter_size, number_of_elements)
        self.hash_seeds: tuple = tuple(HASH_SEEDS[i] for i in range(self.hash_count))
        self.bits = BitArray(self.filter_size)

    def add(self, term):
        """
        Add a value to the index, returns true if the item is new, false if seen before
        """
        bits = self.bits
        term = term.encode()
        for seed in self.hash_seeds:
            hash_ = CityHash32(seed + term) % self.filter_size
            bits.set(hash_, 1)

    def __contains__(self, term):
        bits = self.bits
        term = term.encode()
        for seed in self.hash_seeds:
            hash_ = CityHash32(seed + term) % self.filter_size
            if bits.get(hash_) == 0:
                return False
        return True


if __name__ == "__main__":  # pragma: no cover
    b = BloomFilter()
    import time

    from opteryx.utils import random_string

    n = time.monotonic_ns()
    for i in range(1000000):
        b.add(random_string())
    for i in range(1000000):
        random_string(8) in b
    print((time.monotonic_ns() - n) / 1e9, "opted")



import random

from orso.dataframe import DataFrame
from orso.faker.decimals import generate_random_decimal
from orso.faker.names import generate_random_name
from orso.faker.temporal import generate_random_datetime
from orso.schema import ColumnDisposition
from orso.schema import RelationSchema
from orso.tools import random_int
from orso.tools import random_string
from orso.types import OrsoTypes


def generate_random_row(schema: RelationSchema) -> tuple:
    """
    Generates a random row of values based on the given schema.

    Parameters:
        schema: RelationSchema
            The schema to generate the random row for.

    Returns:
        tuple: A tuple of random values based on the schema.
    """
    columns = [schema.column(i) for i in range(schema.num_columns)]
    row = []
    for column in columns:
        # if the column is nullable, set 1% of the values to null
        if column.nullable and random_int() % 100 == 0:
            row.append(None)
        elif column.type == OrsoTypes.INTEGER:
            if column.disposition == ColumnDisposition.AGE:
                row.append(random.randint(0, 100))
            else:
                row.append(random_int())
        elif column.type == OrsoTypes.VARCHAR:
            if column.disposition == ColumnDisposition.NAME:
                row.append(generate_random_name())
            else:
                row.append(random_string(random_int() % 16 + 8))
        elif column.type == OrsoTypes.BOOLEAN:
            row.append(bool(random.getrandbits(1)))
        elif column.type == OrsoTypes.DECIMAL:
            row.append(generate_random_decimal(column.precision, column.scale))
        elif column.type == OrsoTypes.DOUBLE:
            row.append(random.random())
        elif column.type == OrsoTypes.TIMESTAMP:
            row.append(generate_random_datetime())
        else:
            raise TypeError(f"Orso currently cannot fake {column.type} values.")
    return tuple(row)


def generate_fake_data(schema: RelationSchema, size: int = 100) -> DataFrame:
    """
    Generates a DataFrame of fake data based on the given schema and size.

    Parameters:
        schema: RelationSchema
            The schema to generate the DataFrame for.
        size: int
            The number of rows to generate.

    Returns:
        DataFrame: A DataFrame containing the generated fake data.
    """
    rows = [generate_random_row(schema) for i in range(size)]
    return DataFrame(rows=rows, schema=schema)
